﻿
Imports System.Runtime.InteropServices

Public Class AudioOut

    ' ===============================================================================================
    '  AUDIO OUT - PRIVATE PROPS AND FUNCTIONS
    ' ===============================================================================================
    Private Structure Buffer
        Public Header As WaveNative.WAVEHDR
        Public Data() As Int16
        Public gcHandleHeader As GCHandle
        Public gcHandleData As GCHandle
        Public HeaderPointer As IntPtr
    End Structure
    Private m_Buffers() As Buffer

    Private m_OutputActive As Boolean
    Private m_hWaveOut As Int32
    Private m_Result As Int32
    Private m_BuffersCount As Int32
    Private m_NextBuffer As Int32
    Private m_BufferSize As Int32
    Private m_FillProc As BufferFillEventHandler
    Private m_Timer As MultimediaTimer = New MultimediaTimer

    Private Sub FillBuffers()
        SyncLock Me
            While m_OutputActive
                ' ----------------------------------------------------------------- test the flag WHDR_INQUEUE
                Dim dwFlags As Int16
                dwFlags = Marshal.ReadInt16(m_Buffers(m_NextBuffer).HeaderPointer, 16)
                If (dwFlags And WaveNative.WHDR_INQUEUE) <> 0 Then Exit While
                ' ----------------------------------------------------------------- fill buffer
                m_FillProc(m_Buffers(m_NextBuffer).Data)
                ' ----------------------------------------------------------------- send the buffer
                m_Result = WaveNative.waveOutWrite(m_hWaveOut, _
                                                  m_Buffers(m_NextBuffer).HeaderPointer, _
                                                  WaveNative.WHDR_LEN)
                WaveNative.DecodeError("waveOutWrite", m_Result)
                ' ----------------------------------------------------------------- select next buffer
                m_NextBuffer += 1
                If m_NextBuffer >= m_BuffersCount Then m_NextBuffer = 0
            End While
        End SyncLock
    End Sub

    ' ===============================================================================================
    '  AUDIO OUT - PUBLIC PROPS AND FUNCTIONS
    ' ===============================================================================================
    Friend Delegate Sub BufferFillEventHandler(ByRef data() As Int16)

    Friend Function OutputOn(ByVal device As Integer, _
                             ByVal rate As Int32, _
                             ByVal bits As Int32, _
                             ByVal channels As Int32, _
                             ByVal bufferSize As Integer, _
                             ByVal bufferCount As Integer, _
                             ByVal fillProc As BufferFillEventHandler) As Boolean
        If fillProc Is Nothing Then
            MsgBox("Function OutputOn - You must provide a fillProc.")
            Return False
        End If
        If bits <> 16 Then
            MsgBox("Function OutputOn - Only 16 bit samping is implemented.")
            Return False
        End If
        OutputOff()
        SyncLock Me
            m_FillProc = fillProc
            m_BufferSize = bufferSize
            m_BuffersCount = bufferCount
            Dim format As WaveNative.WAVEFORMATEX
            format = New WaveNative.WAVEFORMATEX(rate, bits, channels)
            ' --------------------------------------------------------------------- Wave Out Open
            m_Result = WaveNative.waveOutOpen(m_hWaveOut, device, format, 0, 0, 0)
            If m_Result <> WaveNative.MMSYSERR_NOERROR Then
                WaveNative.DecodeError("WaveOutOpen", m_Result)
                Return False
            End If
            ' --------------------------------------------------------------------- prepare all the headers 
            ReDim m_Buffers(m_BuffersCount - 1)
            '
            For i As Int32 = 0 To m_BuffersCount - 1
                '
                ReDim m_Buffers(i).Data(bufferSize - 1)
                m_Buffers(i).gcHandleHeader = GCHandle.Alloc(m_Buffers(i).Header, GCHandleType.Pinned)
                m_Buffers(i).gcHandleData = GCHandle.Alloc(m_Buffers(i).Data, GCHandleType.Pinned)
                m_Buffers(i).HeaderPointer = m_Buffers(i).gcHandleHeader.AddrOfPinnedObject()
                m_Buffers(i).Header.lpData = m_Buffers(i).gcHandleData.AddrOfPinnedObject()
                m_Buffers(i).Header.dwBufferLength = m_BufferSize * 2
                m_Buffers(i).Header.dwFlags = 0
                m_Buffers(i).Header.dwLoops = 0
                m_Buffers(i).Header.dwUser = i
                Marshal.StructureToPtr(m_Buffers(i).Header, m_Buffers(i).HeaderPointer, True)
                '
                m_Result = WaveNative.waveOutPrepareHeader(m_hWaveOut, _
                                                           m_Buffers(i).HeaderPointer, _
                                                           WaveNative.WHDR_LEN)
                WaveNative.DecodeError("waveOutPrepareHeader", m_Result)
            Next
            ' --------------------------------------------------------------------- start all
            m_OutputActive = True
            m_NextBuffer = 0
            m_Timer.Timer_Start(10, AddressOf FillBuffers)
            Return True
        End SyncLock
    End Function

    Friend Sub OutputOff()
        SyncLock Me
            m_Timer.Timer_Stop()
            If Not m_OutputActive Then Return
            ' --------------------------------------------------------------------- stop sending buffers
            m_OutputActive = False
            ' --------------------------------------------------------------------- reset
            m_Result = WaveNative.waveOutReset(m_hWaveOut)
            WaveNative.DecodeError("waveOutReset", m_Result)
            ' --------------------------------------------------------------------- wait all the buffers
            ' --------------------------------------------------------------------- wait up to 1 Sec (10 * 100mS)
            For m_NextBuffer = 0 To m_BuffersCount - 1
                For j As Int32 = 1 To 10
                    ' ------------------------------------------------------------- test the flag WHDR_INQUEUE
                    Dim dwFlags As Int16
                    dwFlags = Marshal.ReadInt16(m_Buffers(m_NextBuffer).HeaderPointer, 16)
                    If (dwFlags And WaveNative.WHDR_INQUEUE) = 0 Then Exit For
                    System.Threading.Thread.Sleep(100)
                Next
            Next
            ' --------------------------------------------------------------------- unprepare all the headers 
            For i As Int32 = 0 To m_BuffersCount - 1
                m_Result = WaveNative.waveOutUnprepareHeader(m_hWaveOut, _
                                                             m_Buffers(i).HeaderPointer, _
                                                             WaveNative.WHDR_LEN)
                WaveNative.DecodeError("waveOutUnprepareHeader", m_Result)
            Next
            ' --------------------------------------------------------------------- close wave out
            m_Result = WaveNative.waveOutClose(m_hWaveOut)
            WaveNative.DecodeError("waveOutClose", m_Result)
            ' --------------------------------------------------------------------- free GC Handlers
            For i As Int32 = 0 To m_BuffersCount - 1
                m_Buffers(i).gcHandleHeader.Free()
                m_Buffers(i).gcHandleData.Free()
            Next
        End SyncLock
    End Sub

End Class




