﻿Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False
    Friend Form1_StartWindowState As FormWindowState
    Friend Form2_VisibleAtStart As Boolean
    'Friend Form2_Rectangle As Rectangle

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        If s(2) <> "0" Then
            Return Title & " - V" & s(0) & "." & s(1) & "." & s(2)
        Else
            Return Title & " - V" & s(0) & "." & s(1)
        End If
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 150 - .Width) ' at least at left border + 100 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function
    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function

    Private Function MultilineToString(ByVal s As String) As String
        Return s.Replace(vbCrLf, "%M")
    End Function
    Private Function StringToMultiline(ByVal s As String) As String
        Return s.Replace("%M", vbCrLf)
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub SaveConfigurationAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        sfd.FileName = AssemblyName() & "_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadConfiguration()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
        End If
    End Sub

    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        '
        If Form2 Is Nothing Then Return
        If GetFormRectangle(Form2).Width = 0 Then Return
        '
        If iniFileName = "" Then
            iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
        End If
        '
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Language", Language))
            '
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            r = GetFormRectangle(Form2)
            f.WriteLine(TabString("Form2_Top", r.Top))
            f.WriteLine(TabString("Form2_Left", r.Left))
            f.WriteLine(TabString("Form2_Width", r.Width))
            f.WriteLine(TabString("Form2_Height", r.Height))
            f.WriteLine(TabString("Form2_VisibleAtStart", Form2.Visible And Form2.WindowState <> FormWindowState.Minimized))
            '
            f.WriteLine("")
            f.WriteLine(" Plot controls")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("MeasureUnit", Form1.cmb_MeasureUnit.SelectedIndex))
            f.WriteLine(TabString("FirSeconds", Form1.txt_FirSeconds.NumericValueInteger.ToString))
            f.WriteLine(TabString("ScrollSpeed", Form1.txt_ScrollSpeed.NumericValueInteger.ToString))
            f.WriteLine(TabString("PlotScale", Form1.txt_PlotScale.NumericValue.ToString))
            f.WriteLine(TabString("ShowCandlesticks", Form1.btn_ShowCandlesticks.Checked))
            f.WriteLine(TabString("UserText", MultilineToString(Form1.txt_UserText.Text)))
            '
            f.WriteLine("")
            f.WriteLine(" Sensor props")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SensorType", Form1.cmb_SensorType.Text))
            f.WriteLine(TabString("TubeSens", Form1.txt_TubeSens.NumericValue.ToString))
            f.WriteLine(TabString("TubeBkg", Form1.txt_TubeBkg.NumericValue.ToString))
            f.WriteLine(TabString("TubeDeadTime", Form1.txt_TubeDeadTime.NumericValue.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" InOut Slots")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SlotIn", Form1.txt_SlotIn.Text))
            f.WriteLine(TabString("SlotOut", Form1.txt_SlotOut.Text))
            f.WriteLine(TabString("OutputLinear", Form1.btn_OutputLinear.Checked))
            '
            f.WriteLine("")
            f.WriteLine(" Options")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ClickVolume", Form1.txt_ClickVolume.NumericValueInteger.ToString))
            f.WriteLine(TabString("ClickFrequency", Form1.txt_ClickFrequency.NumericValueInteger.ToString))
            f.WriteLine(TabString("AlarmVolume", Form1.txt_AlarmVolume.NumericValueInteger.ToString))
            f.WriteLine(TabString("AlarmFrequency", Form1.txt_AlarmFrequency.NumericValueInteger.ToString))
            f.WriteLine(TabString("AlarmLevel", Form1.txt_AlarmLevel.NumericValueInteger.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Log")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ExportWithHeader", Form1.chk_ExportWithHeader.Checked))
            f.WriteLine(TabString("DecimalSeparator", Form1.txt_DecimalSeparator.Text))
            f.WriteLine(TabString("FieldSeparator", Form1.txt_FieldSeparator.Text))
            f.WriteLine(TabString("LogToFileTime", Form1.txt_LogToFileTime.NumericValueInteger.ToString))
            f.WriteLine(TabString("LogToFile", Form1.btn_LogToFile.Checked))
            f.WriteLine(TabString("LogFormatCPH", Form1.btn_LogFormatCPH.Checked))
            f.WriteLine(TabString("LogToImagesTime", Form1.txt_LogToImagesTime.NumericValueInteger.ToString))
            f.WriteLine(TabString("LogToImagesImageType", LogToImages_ImageType))
            f.WriteLine(TabString("LogToImages", Form1.btn_LogToImages.Checked))
            f.WriteLine(TabString("GraphAreaOnly", Form1.btn_GraphAreaOnly.Checked))
            f.WriteLine(TabString("ChangeImageNames", Form1.btn_LogChangeNames.Checked))
            f.WriteLine(TabString("SendImagesToFtp", Form1.btn_SendImagesToFtp.Checked))
            f.WriteLine(TabString("SendLogFileToFtp", Form1.btn_SendLogFileToFtp.Checked))
            f.WriteLine(TabString("FtpFolder", Form3.txt_FtpFolder.Text))
            f.WriteLine(TabString("FtpUserName", Form3.txt_FtpUserName.Text))
            f.WriteLine(TabString("FtpPassword", Form3.txt_FtpPassword.Text))
            f.WriteLine(TabString("SendDataToHttp", Form1.btn_SendDataToHttp.Checked))
            f.WriteLine(TabString("HttpUrl", Form3.txt_HttpUrl.Text))
            f.WriteLine(TabString("StationName", Form3.txt_StationName.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Meter")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("MeterScaleType", Form2.GetScaleType))
            f.WriteLine(TabString("MeterSpeed", Form2.GetMeterSpeed.ToString))
            '
            'f.WriteLine("")
            'f.WriteLine(" examples")
            'f.WriteLine("===========================================")
            'f.WriteLine(TabString("DefaultCapMaxDist", Form1.txt_MaxDist.NumericValue))
            'f.WriteLine(TabString("FastScroll", Form2.btn_FastScroll.Checked))
            'f.WriteLine(TabString("ShowRawCount", Form2.btn_ShowRawCount.Checked))
            'f.WriteLine(TabString("ShowXY", Form2.btn_ShowXY.Checked))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub



    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
            LoadAll = True
        End If
        '
        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Language" : Language = l
                        ' -----------------------------------------------------------------
                    Case "Form1_Top" : If LoadAll Then Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : If LoadAll Then Form1.Left = CInt(Val(l))
                    Case "Form1_Width" : If LoadAll Then Form1.Width = CInt(Val(l))
                    Case "Form1_Height" : If LoadAll Then Form1.Height = CInt(Val(l))
                    Case "Form1_WindowState" : If LoadAll Then Form1_StartWindowState = CType(Val(l), FormWindowState)
                        ' -----------------------------------------------------------------
                    Case "Form2_Top" : If LoadAll Then Form2.Top = CInt(Val(l))
                    Case "Form2_Left" : If LoadAll Then Form2.Left = CInt(Val(l))
                    Case "Form2_Width" : If LoadAll Then Form2.Width = CInt(Val(l))
                    Case "Form2_Height" : If LoadAll Then Form2.Height = CInt(Val(l))
                    Case "Form2_VisibleAtStart" : If LoadAll Then Form2_VisibleAtStart = l = "True"
                        ' ----------------------------------------------------------------- Plot controls
                    Case "MeasureUnit" : Form1.cmb_MeasureUnit.SelectedIndex = CInt(Val(l))
                    Case "FirSeconds" : Form1.txt_FirSeconds.NumericValue = Val(l)
                    Case "ScrollSpeed" : Form1.txt_ScrollSpeed.NumericValue = Val(l)
                    Case "PlotScale" : Form1.txt_PlotScale.NumericValue = Val_Double(l)
                    Case "ShowCandlesticks" : Form1.btn_ShowCandlesticks.Checked = l = "True"
                    Case "UserText" : Form1.txt_UserText.Text = StringToMultiline(l)
                        ' ----------------------------------------------------------------- Sensor props
                    Case "SensorType" : Combo_SetIndex_FromString(Form1.cmb_SensorType, l)
                    Case "TubeSens" : Form1.txt_TubeSens.NumericValue = Val_Double(l)
                    Case "TubeBkg" : Form1.txt_TubeBkg.NumericValue = Val_Double(l)
                    Case "TubeDeadTime" : Form1.txt_TubeDeadTime.NumericValue = Val_Double(l)
                        ' ----------------------------------------------------------------- InOut
                    Case "SlotIn" : Form1.txt_SlotIn.Text = l
                    Case "SlotOut" : Form1.txt_SlotOut.Text = l
                    Case "OutputLinear" : Form1.btn_OutputLinear.Checked = l = "True"
                        ' ----------------------------------------------------------------- Options
                    Case "ClickVolume" : Form1.txt_ClickVolume.NumericValue = Val(l)
                    Case "ClickFrequency" : Form1.txt_ClickFrequency.NumericValue = Val(l)
                    Case "AlarmVolume" : Form1.txt_AlarmVolume.NumericValue = Val(l)
                    Case "AlarmFrequency" : Form1.txt_AlarmFrequency.NumericValue = Val(l)
                    Case "AlarmLevel" : Form1.txt_AlarmLevel.NumericValue = Val(l)
                        ' ----------------------------------------------------------------- Log
                    Case "ExportWithHeader" : Form1.chk_ExportWithHeader.Checked = l = "True"
                    Case "DecimalSeparator" : Form1.txt_DecimalSeparator.Text = l
                    Case "FieldSeparator" : Form1.txt_FieldSeparator.Text = l
                    Case "LogToFileTime" : Form1.txt_LogToFileTime.NumericValueInteger = Val_Int(l)
                    Case "LogToFile" : Form1.btn_LogToFile.Checked = l = "True"
                    Case "LogFormatCPH" : Form1.btn_LogFormatCPH.Checked = l = "True"
                    Case "LogToImagesTime" : Form1.txt_LogToImagesTime.NumericValueInteger = Val_Int(l)
                    Case "LogToImagesImageType" : LogToImages_ImageType = l
                    Case "LogToImages" : Form1.btn_LogToImages.Checked = l = "True"
                    Case "GraphAreaOnly" : Form1.btn_GraphAreaOnly.Checked = l = "True"
                    Case "ChangeImageNames" : Form1.btn_LogChangeNames.Checked = l = "True"
                    Case "SendImagesToFtp" : Form1.btn_SendImagesToFtp.Checked = l = "True"
                    Case "SendLogFileToFtp" : Form1.btn_SendLogFileToFtp.Checked = l = "True"
                    Case "FtpFolder" : Form3.txt_FtpFolder.Text = l
                    Case "FtpUserName" : Form3.txt_FtpUserName.Text = l
                    Case "FtpPassword" : Form3.txt_FtpPassword.Text = l
                    Case "SendDataToHttp" : Form1.btn_SendDataToHttp.Checked = l = "True"
                    Case "HttpUrl" : Form3.txt_HttpUrl.Text = l
                    Case "StationName" : Form3.txt_StationName.Text = l
                        ' ----------------------------------------------------------------- Meter
                    Case "MeterScaleType" : Form2.SetScaleType(Val_Int(l))
                    Case "MeterSpeed" : Form2.SetMeterSpeed(Val_Single(l))

                        ' ----------------------------------------------------------------- examples
                        'Case "DefaultCapMaxDist" : Form1.txt_MaxDist.NumericValue = Val(l)
                        'Case "FastScroll" : Form2.btn_FastScroll.Checked = l = "True"
                        'Case "ShowRawCount" : Form2.btn_ShowRawCount.Checked = l = "True"
                        'Case "ShowXY" : Form2.btn_ShowXY.Checked = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
        LimitFormPosition(Form2)

        Select Case Language
            Case "Italian", "English", "Francais", "Deutsch", "Espanol"
            Case Else
                Language = "English"
        End Select
    End Sub


    ' ==================================================================================================
    '  SAVE LOAD -- Sensor data
    ' ==================================================================================================
    Friend SensorData(4, -1) As String

    Friend Sub Load_SensorDataFile()
        Dim filename As String = PlatformAdjustedFileName(Application.StartupPath & "\Extra\Sensor_Data.txt")
        If Not FileExists(filename) Then
            ReinitSensorDataToFactoryDefaults()
            Save_SensorDataFile()
        End If
        If FileExists(filename) Then
            Dim i As Int32 = -1
            Dim s0 As String
            Dim s1() As String
            '
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(filename)
            '
            Do While Not f.EndOfStream
                s0 = f.ReadLine
                s0 = s0.Replace(vbTab, " ")
                s1 = Split(s0, ",")
                If Not s1(0).StartsWith("/") Then
                    If s1.Length = 5 Then
                        i += 1
                        ' --------------------------------------------- TODO restore this - not working with mono (wine OK)
                        'ReDim Preserve SensorData(4, i)
                        RedimPreserve_2D_Array(SensorData, 4, i)
                        ' --------------------------------------------- 
                        SensorData(0, i) = s1(0).Trim
                        SensorData(1, i) = s1(1).Trim
                        SensorData(2, i) = s1(2).Trim
                        SensorData(3, i) = s1(3).Trim
                        SensorData(4, i) = s1(4).Trim
                    End If
                End If
            Loop
            f.Close()
        End If
    End Sub

    Friend Sub Save_SensorDataFile()
        Dim filename As String = PlatformAdjustedFileName(Application.StartupPath & "\Extra\Sensor_Data.txt")
        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(filename))
        Dim f As System.IO.StreamWriter
        f = IO.File.CreateText(filename)
        f.WriteLine("//---------------------------------------------------------------------------")
        f.WriteLine("// Sensor                Sensitivity    Background    Dead time   Sensor     ")
        f.WriteLine("//  Geiger and counters  CPS/mR/h       CPS           uSec        type       ")
        f.WriteLine("//  Ion chambers         CPM/pCi/l                                           ")
        f.WriteLine("//---------------------------------------------------------------------------")
        For i As Int32 = 0 To SensorData.GetLength(1) - 1
            Dim s As String = SensorData(0, i) & "," & SensorData(1, i) & "," & SensorData(2, i) & "," & SensorData(3, i) & "," & SensorData(4, i)
            AlignCsvString(s, New Int32() {24, 39, 53, 65})
            f.WriteLine(s)
        Next
        f.Close()
    End Sub

    Private Sub AlignCsvString(ByRef s As String, ByVal Tabs() As Int32)
        Dim sa() As String
        sa = Split(s, ",")
        s = sa(0).Trim
        For j As Int32 = 1 To sa.Length - 1
            s &= "," & Strings.StrDup(Math.Max(0, Tabs(j - 1) - s.Length), " ") & sa(j).Trim
        Next
    End Sub

    Private Sub AddSensorData(ByVal s1 As String, ByVal s2 As String, ByVal s3 As String, ByVal s4 As String, ByVal s5 As String)
        Dim i As Int32 = SensorData.GetLength(1)
        ' --------------------------------------------- TODO restore this - not working with mono (wine OK)
        'ReDim Preserve SensorData(4, i)
        RedimPreserve_2D_Array(SensorData, 4, i)
        ' --------------------------------------------- 
        SensorData(0, i) = s1
        SensorData(1, i) = s2
        SensorData(2, i) = s3
        SensorData(3, i) = s4
        SensorData(4, i) = s5
    End Sub

    Friend Sub ReinitSensorDataToFactoryDefaults()
        ReDim SensorData(3, -1)
        AddSensorData("4xSBM20", "116", "0.8", "190", "Geiger")
        AddSensorData("SI-8B", "111", "1.0", "100", "Geiger")
        AddSensorData("LND-7312", "60", "0.35", "40", "Geiger")
        AddSensorData("2xSBM20", "58", "0.4", "190", "Geiger")
        AddSensorData("Ludlum 44-9 Pancake", "55", "0.2", "80", "Geiger")
        AddSensorData("2xSI29BG", "36", "0.08", "95", "Geiger")
        AddSensorData("VA-Z-115.1", "32", "0.06", "150", "Geiger")
        AddSensorData("SBM20", "29", "0.2", "190", "Geiger")
        AddSensorData("STS5", "29", "0.16", "190", "Geiger")
        AddSensorData("SI29BG", "18", "0.07", "95", "Geiger")
        AddSensorData("LND-712", "18", "0.1", "90", "Geiger")
        AddSensorData("2xBPW34", "1.2", "0", "0", "Counter")
        AddSensorData("BPW34", "0.6", "0", "0", "Counter")
        AddSensorData("PMT BGO", "2000", "11", "250", "Counter")
        AddSensorData("PMT NaI(Tl)", "4000", "35", "250", "Counter")
        AddSensorData("Chamber A1 (1.0 lt)", "2.22", "0", "10000", "IonChamber")
    End Sub


    ' ==================================================================================================
    '  LogFile
    ' ==================================================================================================
    Private LogFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\Extra\Log.txt")
    Friend LogFile_HeaderRequested As Boolean = True
    Friend LogFile_UseCpmNotCps As Boolean = True

    Friend Sub LogFile_WriteData(ByVal CPS As Single, ByVal value As Single, ByVal valueIsPicoCurie As Boolean)

        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(LogFileName))

        Dim f As System.IO.StreamWriter
        f = My.Computer.FileSystem.OpenTextFileWriter(LogFileName, _
                                                      True, _
                                                      New System.Text.ASCIIEncoding())
        '
        If f.BaseStream.Length = 0 Or LogFile_HeaderRequested Then
            If Form1.chk_ExportWithHeader.Checked AndAlso _
               Form1.btn_LogFormatCPH.Checked = False Then
                LogFile_WriteHeader(f, valueIsPicoCurie)
            Else
                f.WriteLine(vbCrLf & vbCrLf)
            End If
            LogFile_HeaderRequested = False
        End If
        '
        Dim DecimalSeparator As String = Form1.txt_DecimalSeparator.Text
        Dim FieldSeparator As String = Form1.txt_FieldSeparator.Text
        Dim s As String
        s = DateTime.Now.ToShortDateString & " " & _
            DateTime.Now.ToLongTimeString.Replace(".", ":") & FieldSeparator
        If LogFile_UseCpmNotCps Then
            Dim cpm As Single = CPS * 60
            If cpm > 99999 Then
                s += "OverRange"
            Else
                s += cpm.ToString("00000.000", _
                                  Globalization.CultureInfo.InvariantCulture).Replace(".", DecimalSeparator)
            End If
        Else
            If CPS > 99999 Then
                s += "OverRange"
            Else
                s += CPS.ToString("00000.000", _
                                  Globalization.CultureInfo.InvariantCulture).Replace(".", DecimalSeparator)
            End If
        End If
        s += FieldSeparator
        If value > 99999 Then
            s += "OverRange"
        Else
            s += value.ToString("00000.000", _
                                Globalization.CultureInfo.InvariantCulture).Replace(".", DecimalSeparator)
        End If
        f.WriteLine(s)
        f.Close()
    End Sub

    Private Sub LogFile_WriteHeader(ByVal f As IO.StreamWriter, ByVal valueIsPicoCurie As Boolean)
        f.WriteLine(vbCrLf)
        f.WriteLine("# -------------------------------------")
        f.WriteLine("#  Session start: " & Date.Now.ToString("yyyy/MM/dd HH':'mm':'ss"))
        f.WriteLine("# -------------------------------------")
        f.WriteLine("#       FIR Seconds: " & Form1.txt_FirSeconds.Text)
        f.WriteLine("#       Sensor type: " & Form1.cmb_SensorType.Text)
        f.WriteLine("#      Sensor Sens.: " & Form1.txt_TubeSens.Text)
        f.WriteLine("#        Sensor BKG: " & Form1.txt_TubeBkg.Text)
        f.WriteLine("#  Sensor Dead-time: " & Form1.txt_TubeDeadTime.Text)
        f.WriteLine("# -------------------------------------")
        If valueIsPicoCurie Then
            If LogFile_UseCpmNotCps Then
                f.WriteLine("#  DATE      TIME      CPM     pCi/l")
            Else
                f.WriteLine("#  DATE      TIME      CPS     pCi/l")
            End If
        Else
            If LogFile_UseCpmNotCps Then
                f.WriteLine("#  DATE      TIME      CPM      uS/h")
            Else
                f.WriteLine("#  DATE      TIME      CPS      uS/h")
            End If
        End If
        f.WriteLine("# -------------------------------------")
    End Sub

    Friend Sub LogFile_WriteCPH(ByVal CPH As Single)

        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(LogFileName))

        Dim f As System.IO.StreamWriter
        f = My.Computer.FileSystem.OpenTextFileWriter(LogFileName, _
                                                      True, _
                                                      New System.Text.ASCIIEncoding())
        '
        Dim DecimalSeparator As String = Form1.txt_DecimalSeparator.Text
        Dim s As String
        s = DateTime.Now.ToShortDateString & " " & _
            DateTime.Now.ToLongTimeString.Replace(".", ":") & " " & _
            CPH.ToString("0")
        f.WriteLine(s)
        f.Close()
    End Sub


    ' ==================================================================================================
    '  LogFile
    ' ==================================================================================================
    Friend LogToImages_ImageType As String
    Private LogToImageFileName As String
    Private FtpFile As String
    Private FtpUserName As String
    Private FtpPassword As String
    Friend Sub LogToImages_WriteImage(ByVal GraphAreaOnly As Boolean)
        LogToImageFileName = Application.StartupPath & "\Extra\Images\"

        If Form1.btn_LogChangeNames.Checked Then
            LogToImageFileName &= "PLOT_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        Else
            LogToImageFileName &= "PLOT_LOG"
        End If
        LogToImageFileName &= "." & LogToImages_ImageType

        LogToImageFileName = PlatformAdjustedFileName(LogToImageFileName)
        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(LogToImageFileName))

        Try
            If GraphAreaOnly Then
                SaveImage(GetImage(Form1.GroupBox_RadiationPlot), LogToImageFileName, LogToImages_ImageType, 100)
            Else
                SaveImage(GetClientAreaImage(), LogToImageFileName, LogToImages_ImageType, 100)
            End If

            If Form1.btn_SendImagesToFtp.Checked Then
                SendImageToFtp_ASYNC()
            End If
        Catch
        End Try
    End Sub

    Friend Sub SendImageToFtp_ASYNC()
        ' --------------------------------------------------------- prepare vars before to start the thread
        FtpFile = Form3.txt_FtpFolder.Text & "/" & _
                  IO.Path.GetFileName(LogToImageFileName)
        FtpUserName = Form3.txt_FtpUserName.Text
        FtpPassword = Form3.txt_FtpPassword.Text
        ' --------------------------------------------------------- asynchronously start the Thread 
        Dim objThread As Threading.Thread
        objThread = New Threading.Thread(AddressOf SendImageToFtp)
        ' --------------------------------------------------------- make the thread as background thread.
        objThread.IsBackground = True
        ' --------------------------------------------------------- set the Priority of the thread.
        objThread.Priority = Threading.ThreadPriority.BelowNormal
        ' --------------------------------------------------------- start the thread.
        objThread.Start()
    End Sub

    Private Sub SendImageToFtp()
        FTP_Upload(LogToImageFileName, FtpFile, FtpUserName, FtpPassword)
    End Sub


    ' ==================================================================================================
    '  SEND LogFile to FTP
    ' ==================================================================================================
    Private LogFileSendToFtp_FtpFile As String
    Private LogFileSendToFtp_LocalFile As String

    Friend Sub LogFileSendToFtp_ASYNC()
        ' --------------------------------------------------------- prepare vars before to start the thread
        Dim logfile As String = PlatformAdjustedFileName(Application.StartupPath & "\Extra\Log.txt")
        LogFileSendToFtp_LocalFile = PlatformAdjustedFileName(Application.StartupPath & "\Extra\Log2.txt")
        LogFileSendToFtp_FtpFile = Form3.txt_FtpFolder.Text & "/Log.txt"
        FtpUserName = Form3.txt_FtpUserName.Text
        FtpPassword = Form3.txt_FtpPassword.Text
        ' --------------------------------------------------------- 
        Try
            IO.File.Copy(logfile, LogFileSendToFtp_LocalFile, True)
        Catch
        End Try
        ' --------------------------------------------------------- asynchronously start the Thread 
        Dim objThread As Threading.Thread
        objThread = New Threading.Thread(AddressOf LogFile_SendToFtp)
        ' --------------------------------------------------------- make the thread as background thread.
        objThread.IsBackground = True
        ' --------------------------------------------------------- set the Priority of the thread.
        objThread.Priority = Threading.ThreadPriority.BelowNormal
        ' --------------------------------------------------------- start the thread.
        objThread.Start()
    End Sub
    Friend Sub LogFile_SendToFtp()
        Try
            FTP_Upload(LogFileSendToFtp_LocalFile, LogFileSendToFtp_FtpFile, FtpUserName, FtpPassword)
        Catch
        End Try
    End Sub

    ' ==================================================================================================
    '  SEND last data to HTTP
    ' ==================================================================================================
    Private httpUrl As String
    Private httpString As String
    Friend Sub SendLastDataToHttp_ASYNC(ByVal CPS As Single, ByVal value As Single)
        httpUrl = Form3.txt_HttpUrl.Text                          ' "http://www.theremino.com/log_to_server.php"
        Dim stationString As String = Form3.txt_StationName.Text  ' "Cascinette d'Ivrea - Station 2334 - GMT+1"
        Dim s As String
        s = "station="
        s &= stationString.Replace("&", "")
        s = s.Replace("""", "")
        s &= "&date=" & DateTime.Now.ToShortDateString
        s &= "&time=" & DateTime.Now.ToLongTimeString
        s += "&cpm=" & (CPS * 60).ToString("0.000", Globalization.CultureInfo.InvariantCulture)
        s += "&value=" & value.ToString("0.000", Globalization.CultureInfo.InvariantCulture)
        httpString = s
        ' --------------------------------------------------------- asynchronously start the Thread 
        Dim objThread As Threading.Thread
        objThread = New Threading.Thread(AddressOf SendToHttp)
        ' --------------------------------------------------------- make the thread as background thread.
        objThread.IsBackground = True
        ' --------------------------------------------------------- set the Priority of the thread.
        objThread.Priority = Threading.ThreadPriority.BelowNormal
        ' --------------------------------------------------------- start the thread.
        objThread.Start()
    End Sub
    Public Sub SendToHttp()
        Try
            Dim request As System.Net.WebRequest = System.Net.WebRequest.Create(httpUrl)
            request.Method = "POST"
            Dim byteArray As Byte() = System.Text.Encoding.UTF8.GetBytes(httpString)
            request.ContentType = "application/x-www-form-urlencoded"
            request.ContentLength = byteArray.Length
            Dim dataStream As IO.Stream = request.GetRequestStream()
            dataStream.Write(byteArray, 0, byteArray.Length)
            dataStream.Close()
            ' --------------------------------------------------------- response from server
            Dim response As Net.WebResponse = request.GetResponse()
            dataStream = response.GetResponseStream()
            Dim reader As New IO.StreamReader(dataStream)
            Dim responseFromServer As String = reader.ReadToEnd()
            reader.Close()
            dataStream.Close()
            response.Close()
        Catch ex As Exception
            'Dim error1 As String = ErrorToString()
            'If error1 = "Invalid URI: The format of the URI could not be determined." Then
            '    error1 = "ERROR! Must have HTTP:// before the URL."
            'End If
            'MsgBox(error1)
        End Try
    End Sub


    ' ==================================================================================================
    '  SAVE IMAGE
    ' ==================================================================================================
    Public Sub SaveImage(ByVal img As Image, _
                         ByVal filename As String, _
                         ByVal extension As String, _
                         ByVal Quality As Int32)

        extension = LCase(extension)
        filename = RemoveExtension(filename)
        filename += "." & extension
        '
        If img Is Nothing Then Exit Sub
        Try
            File_Kill(filename)
            If extension = "jpg" Then
                Dim ImageEncoders() As ImageCodecInfo = ImageCodecInfo.GetImageEncoders()
                Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
                Dim myEncoderParameters As New EncoderParameters(1)
                Dim myEncoderParameter As New EncoderParameter(myEncoder, 100)
                myEncoderParameters.Param(0) = myEncoderParameter
                img.Save(filename, ImageEncoders(1), myEncoderParameters)
            Else
                img.Save(filename, ImageFormatFromFileExtension(extension))
            End If
        Catch
            MsgBox("Image save error", MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Function ImageFormatFromFileExtension(ByVal extension As String) As ImageFormat
        Select Case LCase(extension)
            Case "jpg" : Return ImageFormat.Jpeg
            Case "png" : Return ImageFormat.Png
            Case "tiff" : Return ImageFormat.Tiff
            Case "exif" : Return ImageFormat.Exif
            Case "emf" : Return ImageFormat.Emf
            Case "wmf" : Return ImageFormat.Wmf
            Case "gif" : Return ImageFormat.Gif
            Case "bmp" : Return ImageFormat.Bmp
                'Case "ico" : Return ImageFormat.Icon
            Case Else : Return ImageFormat.Jpeg
        End Select
    End Function

    Friend Sub File_Kill(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            My.Computer.FileSystem.DeleteFile(filename, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)
        End If
    End Sub

    Friend Sub SaveImage(ByVal Prefix As String, ByVal cnt As Control)
        Dim img As Image = GetImage(cnt)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Portable network graphics (*.png)|*png|JPEG image (*.jpg)|*.jpg"
        sfd.FileName = Prefix & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveImage(img, sfd.FileName, If(sfd.FilterIndex = 2, "jpg", "png"), 100)
        End If
    End Sub
    Friend Function GetImage(ByVal cnt As Control) As Bitmap
        Dim s As Size = cnt.Size
        Dim bmp As Bitmap = New Bitmap(s.Width, s.Height)
        cnt.DrawToBitmap(bmp, New Rectangle(0, 0, s.Width, s.Height))
        Return bmp
    End Function

    Friend Function GetClientAreaImage() As Bitmap
        Dim cnt As Form = Form1
        Dim posy As Int32 = Form1.MenuStrip1.Height + Form1.ToolStrip1.Height

        ' -------------------------------------------- calc delta to client area upper left
        Dim origin As Point = cnt.PointToScreen(New Point(0, 0))
        Dim dx As Integer = origin.X - cnt.Left
        Dim dy As Integer = origin.Y - cnt.Top + posy

        ' -------------------------------------------- Copy the client area into a new Bitmap
        Dim w As Integer = cnt.ClientSize.Width
        Dim h As Integer = cnt.ClientSize.Height - posy
        Dim bm As New Bitmap(w, h)
        Graphics.FromImage(bm).DrawImage(GetImage(cnt), 0, 0, _
                                         New Rectangle(dx, dy, w, h), _
                                         GraphicsUnit.Pixel)
        Return bm
    End Function

    ' ==================================================================================================
    '  SAVE CPM TABLE
    ' ==================================================================================================
    Friend Sub SaveCpmTable()
        Dim filename As String = PlatformAdjustedFileName(Application.StartupPath & "\Extra\CPM_Table.txt")
        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(filename))
        Dim f As System.IO.StreamWriter
        f = IO.File.CreateText(filename)
        f.WriteLine("---------------------------------------------")
        f.WriteLine(" Table created by Theremino-Geiger")
        '
        Dim fp As System.IFormatProvider = Globalization.CultureInfo.InvariantCulture
        Dim s As String
        If Form1.cmb_SensorType.Text = "IonChamber" Then
            f.WriteLine("---------------------------------------------")
            f.WriteLine(" Sensor type            : Ion chamber")
            f.WriteLine(" Sensitivity  (CPS/Bq/l): " & Form1.Tube_Sens.ToString("0.00", fp))
            f.WriteLine(" Background        (CPS): " & Form1.Tube_Bkg.ToString("0.00", fp))
            f.WriteLine(" Dead time        (uSec): " & Form1.Tube_DeadTime.ToString("0"))
            f.WriteLine("---------------------------------------------")
            f.WriteLine("")
            f.WriteLine("")
            f.WriteLine(" CPM          pCi/l         Bq/m3")
            f.WriteLine("---------------------------------------------")
            '
            Dim CpmArray() As Int32 = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 20, 30, 50, 80, 100, 120, 150, 200, 250}
            Dim local_pCi As Single
            Dim local_Bq As Single
            For Each i As Int32 In CpmArray
                local_pCi = Form1.CalcPicoCurie(i / 60.0F)
                local_Bq = Form1.CalcBequerels(i / 60.0F)
                If local_Bq < 0.1 Then local_Bq = 0.1
                If local_pCi < 0.01 Then local_pCi = 0.01
                '
                s = "," & i.ToString("0") & "," & local_pCi.ToString("0.0", fp) & "," & local_Bq.ToString("0", fp)
                AlignCsvString(s, New Int32() {1, 15, 30})
                f.WriteLine(s.Replace(",", ""))
            Next
            '
            f.WriteLine("")
            f.WriteLine("")
            f.WriteLine(" Ambient background : 5 to 10 Bq/m3 (0.2 to 0.4 pCi/l)")
            f.WriteLine(" Attention level    : 200 Bq/m3 (5 pCi/l)")
            f.WriteLine(" Danger level       : 2000 Bq/m3 (50 pCi/l)")
        Else
            f.WriteLine("---------------------------------------------")
            f.WriteLine(" Sensor type           : Geiger tube")
            f.WriteLine(" Sensitivity (CPS/mR/h): " & Form1.Tube_Sens.ToString("0.0", fp))
            f.WriteLine(" Background       (CPS): " & Form1.Tube_Bkg.ToString("0.0", fp))
            f.WriteLine(" Dead time       (uSec): " & Form1.Tube_DeadTime.ToString("0"))
            f.WriteLine("---------------------------------------------")
            f.WriteLine("")
            f.WriteLine("")
            f.WriteLine(" CPM          uS/h          mR/h")
            f.WriteLine("---------------------------------------------")
            '
            Dim CpmArray() As Int32 = {10, 12, 15, 18, 20, 25, 30, 40, 50, 60, 70, 80, 90, 100, 120, 150, 200, 250, 300, 500}
            Dim local_uRh As Single
            Dim local_uSh As Single
            For Each i As Int32 In CpmArray
                local_uRh = Form1.CalcMicroRoentgens(i / 60.0F)
                local_uSh = Form1.CalcMicroSieverts(local_uRh)
                If local_uRh < 1 Then local_uRh = 1
                If local_uSh < 0.01 Then local_uSh = 0.01
                '
                s = "," & i.ToString("0") & "," & local_uSh.ToString("0.00", fp) & "," & local_uRh.ToString("0", fp)
                AlignCsvString(s, New Int32() {1, 15, 30})
                f.WriteLine(s.Replace(",", ""))
            Next
            '
            f.WriteLine("")
            f.WriteLine("")
            f.WriteLine(" Ambient background : 8 to 20 mR/h (0.08 to 0.2 uS/h)")
            f.WriteLine(" Attention level    : 100 mR/h (1 uS/h)")
            f.WriteLine(" Danger level       : 1000 mR/h (10 uS/h)")
        End If
        f.WriteLine("")
        f.WriteLine(" For more precision at low radiation levels")
        f.WriteLine(" count for ten minutes and divide CPM by 10")
        '
        f.Close()
    End Sub

End Module
